// Label3D.cc
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "Label3D.hh"

Label3D::Label3D(pfGroup *parent){
  string=new pfString;
  font=pfdLoadFont_type1("/usr/share/Performer/data/Times-Elfin",PFDFONT_FILLED);
  string->setFont(font);
  string->setString("");
  text=new pfText;
  text->addString(string);
  if(parent)
    parent->addChild(text);
}

Label3D::~Label3D(){
  string->unrefDelete();
  font->checkDelete();
  text->unrefDelete();
}

void Label3D::setLabel(char *label){
  string->setString(label); // assuming the string is still around
}
void Label3D::setFont(char *fontname,FontStyle style){
  // should I delete the old font?
  pfFont *oldfont=font;
  if(!strcmp(fontname,"Times") || 
     !strcmp(fontname,"Times-Elfin")){
    // load times-elfin
    font=pfdLoadFont_type1("/usr/share/Performer/data/Times-Elfin",style);
  }
  else if(!strcmp(fontname,"Mistr")){
    font=pfdLoadFont_type1("/usr/share/Performer/data/Mistr",style);
  }
  else
    // load direct
    font=pfdLoadFont_type1(fontname,style);
  if(font){
    string->setFont(font);
    oldfont->checkDelete();
  }
  else font=oldfont; // return to original
}

// not acceptable... need to be able to retransform from scratch
// or get the scale and translation from the mat
void Label3D::setFontSize(float height){
  // std height of the font = 1
  pfMatrix mat;
  getMat(mat);
  mat.postScale(mat,height,height,height);
  setMat(mat);
}

void Label3D::getSize(float &width,float &height){
  const pfBox *box=string->getBBox();
  pfVec3 size = box->max - box->min;
  // width and height
  width=size[0];
  height=size[2];
}

void Label3D::setSize(float width,float height){
  pfMatrix mat;
  getMat(mat);
  if(width<0){
    if(height>0)
      mat.postScale(mat,height,height,height);
  }
  else if(height<0){
    if(width>0)
      mat.postScale(mat,width,width,width);
  }
  else
    mat.postScale(mat,width,1.0,height);
  setMat(mat);
}

void Label3D::setTrans(float x,float y,float z){
  pfMatrix mat;
  getMat(mat);
  mat.postTrans(mat,x,y,z);
  setMat(mat);	
}
void Label3D::setRot(float deg,float x,float y,float z){
  pfMatrix mat;
  getMat(mat);
  mat.postRot(mat,deg,x,y,z);
  setMat(mat);
}
